@@ -1,28 +1,18 @@
 import html
-import json
-from pprint import pprint
+from concurrent.futures import ThreadPoolExecutor, as_completed
+from beartype.typing import List, Union, Tuple, Dict
 
 from trcli.api.api_client import APIClient, APIClientResult
-from trcli.cli import Environment
 from trcli.api.api_response_verify import ApiResponseVerify
-from trcli.data_classes.dataclass_testrail import TestRailSuite, TestRailCase
-from trcli.data_classes.data_parsers import MatchersParser
-from trcli.data_providers.api_data_provider import ApiDataProvider
+from trcli.cli import Environment
 from trcli.constants import (
     ProjectErrors,
     FAULT_MAPPING,
 )
+from trcli.data_classes.data_parsers import MatchersParser
+from trcli.data_classes.dataclass_testrail import TestRailSuite, TestRailCase, ProjectData
+from trcli.data_providers.api_data_provider import ApiDataProvider
 from trcli.settings import MAX_WORKERS_ADD_RESULTS, MAX_WORKERS_ADD_CASE
-from typing import List, Union
-from dataclasses import dataclass
-from concurrent.futures import ThreadPoolExecutor, as_completed
-
-
-@dataclass
-class ProjectData:
-    project_id: int
-    suite_mode: int
-    error_message: str
 
 
 class ApiRequestHandler:
@@ -38,7 +28,13 @@ class ApiRequestHandler:
         self.environment = environment
         self.client = api_client
         self.suffix = api_client.VERSION
-        self.data_provider = ApiDataProvider(suites_data, environment.case_fields, environment.run_description, environment.result_fields)
+        self.data_provider = ApiDataProvider(
+            suites_data,
+            environment.case_fields,
+            environment.run_description,
+            environment.result_fields,
+            environment.section_id
+        )
         self.suites_data_from_provider = self.data_provider.suites_input
         self.response_verifier = ApiResponseVerify(verify)
 
@@ -50,7 +46,7 @@ class ApiRequestHandler:
         """
         response = self.client.send_get("get_case_fields")
         if not response.error_message:
-            fields: list = response.response_text
+            fields: List = response.response_text
             automation_id_field = next(
                 filter(lambda x: x["system_name"] == "custom_automation_id", fields),
                 None
@@ -139,7 +135,7 @@ class ApiRequestHandler:
         else:
             return None, response.error_message
 
-    def resolve_suite_id_using_name(self, project_id: int) -> (int, str):
+    def resolve_suite_id_using_name(self, project_id: int) -> Tuple[int, str]:
         """Get suite ID matching suite name on data provider or returns -1 if unable to match any suite.
         :arg project_id: project id
         :returns: tuple with id of the suite and error message"""
@@ -160,7 +156,7 @@ class ApiRequestHandler:
 
         return suite_id, error_message
 
-    def get_suite_ids(self, project_id: int) -> (List[int], str):
+    def get_suite_ids(self, project_id: int) -> Tuple[List[int], str]:
         """Get suite IDs for requested project_id.
         : project_id: project id
         : returns: tuple with list of suite ids and error string"""
@@ -170,7 +166,7 @@ class ApiRequestHandler:
         response = self.client.send_get(f"get_suites/{project_id}")
         if not response.error_message:
             for suite in response.response_text:
-                available_suites.append(suite["id"])
+                available_suites.append(int(suite["id"]))
                 returned_resources.append(
                     {
                         "suite_id": suite["id"],
@@ -185,7 +181,7 @@ class ApiRequestHandler:
         ) > 0 else "Update skipped"
         return available_suites, error_message
 
-    def add_suites(self, project_id: int) -> (List[dict], str):
+    def add_suites(self, project_id: int) -> Tuple[List[Dict], str]:
         """
         Adds suites that doesn't have ID's in DataProvider.
         Runs update_data in data_provider for successfully created resources.
@@ -221,7 +217,7 @@ class ApiRequestHandler:
         ) > 0 else "Update skipped"
         return returned_resources, error_message
 
-    def check_missing_section_ids(self, project_id: int) -> (List[int], str):
+    def check_missing_section_ids(self, project_id: int) -> Tuple[bool, str]:
         """
         Check what section id's are missing in DataProvider.
         :project_id: project_id
@@ -248,7 +244,7 @@ class ApiRequestHandler:
         else:
             return False, error_message
 
-    def add_sections(self, project_id: int) -> (List[dict], str):
+    def add_sections(self, project_id: int) -> Tuple[List[Dict], str]:
         """
         Add sections that doesn't have ID in DataProvider.
         Runs update_data in data_provider for successfully created resources.
@@ -284,7 +280,7 @@ class ApiRequestHandler:
         ) > 0 else "Update skipped"
         return returned_resources, error_message
 
-    def check_missing_test_cases_ids(self, project_id: int) -> (bool, str):
+    def check_missing_test_cases_ids(self, project_id: int) -> Tuple[bool, str]:
         """
         Check what test cases id's are missing in DataProvider.
         :project_id: project_id
@@ -334,7 +330,7 @@ class ApiRequestHandler:
 
         return missing_cases_number > 0, ""
 
-    def add_cases(self) -> (List[dict], str):
+    def add_cases(self) -> Tuple[List[dict], str]:
         """
         Add cases that doesn't have ID in DataProvider.
         Runs update_data in data_provider for successfully created resources.
@@ -371,7 +367,14 @@ class ApiRequestHandler:
         ]
         return returned_resources, error_message
 
-    def add_run(self, project_id: int, run_name: str, milestone_id: int = None) -> (List[dict], str):
+    def add_run(
+            self,
+            project_id: int,
+            run_name: str,
+            milestone_id: int = None,
+            plan_id: int = None,
+            config_ids: List[int] = None
+    ) -> Tuple[int, str]:
         """
         Creates a new test run.
         :project_id: project_id
@@ -379,10 +382,60 @@ class ApiRequestHandler:
         :returns: Tuple with run id and error string.
         """
         add_run_data = self.data_provider.add_run(run_name, milestone_id=milestone_id)
-        response = self.client.send_post(f"add_run/{project_id}", add_run_data)
-        return response.response_text.get("id"), response.error_message
+        if not plan_id:
+            response = self.client.send_post(f"add_run/{project_id}", add_run_data)
+            run_id = response.response_text.get("id")
+        else:
+            if config_ids:
+                add_run_data["config_ids"] = config_ids
+                entry_data = {
+                    "name": add_run_data["name"],
+                    "suite_id": add_run_data["suite_id"],
+                    "config_ids": config_ids,
+                    "runs": [add_run_data]
+                }
+            else:
+                entry_data = add_run_data
+            response = self.client.send_post(f"add_plan_entry/{plan_id}", entry_data)
+            run_id = response.response_text["runs"][0]["id"]
+        return run_id, response.error_message
 
-    def upload_attachments(self, report_results: [dict], results: list[dict], run_id: int):
+    def update_run(self, run_id: int, run_name: str, milestone_id: int = None) -> Tuple[dict, str]:
+        """
+        Updates an existing run
+        :run_id: run id
+        :run_name: run name
+        :returns: Tuple with run and error string.
+        """
+        add_run_data = self.data_provider.add_run(run_name, milestone_id=milestone_id)
+        run_tests, error_message = self.__get_all_tests_in_run(run_id)
+        run_case_ids = [test["case_id"] for test in run_tests]
+        report_case_ids = add_run_data["case_ids"]
+        joint_case_ids = list(set(report_case_ids + run_case_ids))
+        add_run_data["case_ids"] = joint_case_ids
+        run_response = self.client.send_get(f"get_run/{run_id}")
+        plan_id = run_response.response_text["plan_id"]
+        config_ids = run_response.response_text["config_ids"]
+        if not plan_id:
+            update_response = self.client.send_post(f"update_run/{run_id}", add_run_data)
+        elif plan_id and config_ids:
+            update_response = self.client.send_post(f"update_run_in_plan_entry/{run_id}", add_run_data)
+        else:
+            response = self.client.send_get(f"get_plan/{plan_id}")
+            entry_id = next(
+                (
+                    run["entry_id"]
+                    for entry in response.response_text["entries"]
+                    for run in entry["runs"]
+                    if run["id"] == run_id
+                ),
+                None,
+            )
+            update_response = self.client.send_post(f"update_plan_entry/{plan_id}/{entry_id}", add_run_data)
+        run_response = self.client.send_get(f"get_run/{run_id}")
+        return run_response.response_text, update_response.error_message
+
+    def upload_attachments(self, report_results: [Dict], results: List[Dict], run_id: int):
         """ Getting test result id and upload attachments for it. """
         tests_in_run, error = self.__get_all_tests_in_run(run_id)
         if not error:
@@ -399,7 +452,7 @@ class ApiRequestHandler:
         else:
             self.environment.elog(f"Unable to upload attachments due to API request error: {error}")
 
-    def add_results(self, run_id: int) -> (dict, str):
+    def add_results(self, run_id: int) -> Tuple[List, str, int]:
         """
         Adds one or more new test results.
         :run_id: run id
@@ -455,7 +508,7 @@ class ApiRequestHandler:
             self.environment.log(f"No attachments found to upload.")
         return responses, error_message, progress_bar.n
 
-    def handle_futures(self, futures, action_string, progress_bar):
+    def handle_futures(self, futures, action_string, progress_bar) -> Tuple[list, str]:
         responses = []
         error_message = ""
         try:
@@ -492,7 +545,7 @@ class ApiRequestHandler:
             raise KeyboardInterrupt
         return responses, error_message
 
-    def close_run(self, run_id: int) -> (dict, str):
+    def close_run(self, run_id: int) -> Tuple[dict, str]:
         """
         Closes an existing test run and archives its tests & results.
         :run_id: run id
@@ -502,7 +555,7 @@ class ApiRequestHandler:
         response = self.client.send_post(f"close_run/{run_id}", body)
         return response.response_text, response.error_message
 
-    def delete_suite(self, suite_id: int) -> (dict, str):
+    def delete_suite(self, suite_id: int) -> Tuple[dict, str]:
         """
         Delete suite given suite id
         :suite_id: suite id
@@ -511,7 +564,7 @@ class ApiRequestHandler:
         response = self.client.send_post(f"delete_suite/{suite_id}", payload={})
         return response.response_text, response.error_message
 
-    def delete_sections(self, added_sections: List[dict]) -> (dict, str):
+    def delete_sections(self, added_sections: List[Dict]) -> Tuple[List, str]:
         """
         Delete section given add_sections response
         :suite_id: section id
@@ -530,7 +583,7 @@ class ApiRequestHandler:
                 break
         return responses, error_message
 
-    def delete_cases(self, suite_id: int, added_cases: List[dict]) -> (dict, str):
+    def delete_cases(self, suite_id: int, added_cases: List[Dict]) -> Tuple[Dict, str]:
         """
         Delete cases given add_cases response
         :suite_id: section id
@@ -540,7 +593,7 @@ class ApiRequestHandler:
         response = self.client.send_post(f"delete_cases/{suite_id}", payload=body)
         return response.response_text, response.error_message
 
-    def delete_run(self, run_id) -> (dict, str):
+    def delete_run(self, run_id) -> Tuple[dict, str]:
         """
         Delete run given add_run response
         :suite_id: section id
@@ -550,7 +603,7 @@ class ApiRequestHandler:
         return response.response_text, response.error_message
 
     @staticmethod
-    def retrieve_results_after_cancelling(futures):
+    def retrieve_results_after_cancelling(futures) -> list:
         responses = []
         for future in as_completed(futures):
             if not future.cancelled():
@@ -559,9 +612,9 @@ class ApiRequestHandler:
                     responses.append(response)
         return responses
 
-    def _add_case_and_update_data(self, case: TestRailCase):
+    def _add_case_and_update_data(self, case: TestRailCase) -> APIClientResult:
         case_body = case.to_dict()
-        if self.environment.case_matcher != MatchersParser.AUTO:
+        if self.environment.case_matcher != MatchersParser.AUTO and "custom_automation_id" in case_body:
             case_body.pop("custom_automation_id")
         response = self.client.send_post(f"add_case/{case_body.pop('section_id')}", case_body)
         if response.status_code == 200:
@@ -577,31 +630,31 @@ class ApiRequestHandler:
         for future in futures:
             future.cancel()
 
-    def __get_all_cases(self, project_id=None, suite_id=None) -> (List[dict], str):
+    def __get_all_cases(self, project_id=None, suite_id=None) -> Tuple[List[dict], str]:
         """
         Get all cases from all pages
         """
         return self.__get_all_entities('cases', f"get_cases/{project_id}&suite_id={suite_id}")
 
-    def __get_all_sections(self, project_id=None, suite_id=None) -> (List[dict], str):
+    def __get_all_sections(self, project_id=None, suite_id=None) -> Tuple[List[dict], str]:
         """
         Get all sections from all pages
         """
         return self.__get_all_entities('sections', f"get_sections/{project_id}&suite_id={suite_id}")
 
-    def __get_all_tests_in_run(self, run_id=None) -> (List[dict], str):
+    def __get_all_tests_in_run(self, run_id=None) -> Tuple[List[dict], str]:
         """
         Get all tests from all pages
         """
         return self.__get_all_entities('tests', f"get_tests/{run_id}")
 
-    def __get_all_projects(self) -> (List[dict], str):
+    def __get_all_projects(self) -> Tuple[List[dict], str]:
         """
         Get all cases from all pages
         """
         return self.__get_all_entities('projects', f"get_projects")
 
-    def __get_all_entities(self, entity: str, link=None, entities=[]) -> (List[dict], str):
+    def __get_all_entities(self, entity: str, link=None, entities=[]) -> Tuple[List[Dict], str]:
         """
         Get all entities from all pages if number of entities is too big to return in single response.
         Function using next page field in API response.
