@@ -1,5 +1,5 @@
 import time
-from typing import Tuple, Callable, List
+from beartype.typing import Tuple, Callable, List
 
 from trcli.api.api_client import APIClient
 from trcli.api.api_request_handler import ApiRequestHandler
@@ -16,25 +16,21 @@ class ResultsUploader:
     Initialized with environment object and result file parser object (any parser derived from FileParser).
     """
 
-    def __init__(self, environment: Environment, suite: TestRailSuite):
+    def __init__(self, environment: Environment, suite: TestRailSuite, skip_run: bool = False):
         self.project = None
         self.environment = environment
-        self.parsed_data = suite
+        self.skip_run = skip_run
         self.run_name = self.environment.title
         if self.environment.special_parser == "saucectl":
-            self.run_name += f" ({self.parsed_data.name})"
+            self.run_name += f" ({suite.name})"
         if self.environment.suite_id:
-            self.parsed_data.suite_id = self.environment.suite_id
+            suite.suite_id = self.environment.suite_id
         self.api_request_handler = ApiRequestHandler(
             api_client=self.instantiate_api_client(),
             environment=self.environment,
-            suites_data=self.parsed_data,
+            suites_data=suite,
             verify=self.environment.verify,
         )
-        if self.environment.suite_id:
-            self.api_request_handler.data_provider.update_data(
-                [{"suite_id": self.environment.suite_id}]
-            )
 
     def upload_results(self):
         """
@@ -61,7 +57,7 @@ class ResultsUploader:
         self.environment.log("Done.")
 
         # Resolve test suite
-        added_suite_id, result_code = self.get_suite_id(
+        suite_id, result_code, suite_added = self.get_suite_id(
             project_id=self.project.project_id, suite_mode=self.project.suite_mode
         )
         if result_code == -1:
@@ -85,7 +81,7 @@ class ResultsUploader:
             )
             if result_code == -1:
                 revert_logs = self.rollback_changes(
-                    added_suite_id=added_suite_id, added_sections=added_sections
+                    suite_id=suite_id, suite_added=suite_added, added_sections=added_sections
                 )
                 self.environment.log("\n".join(revert_logs))
                 exit(1)
@@ -96,38 +92,70 @@ class ResultsUploader:
                 result_code = 1
             if result_code == -1:
                 revert_logs = self.rollback_changes(
-                    added_suite_id=added_suite_id,
+                    suite_id=suite_id,
+                    suite_added=suite_added,
                     added_sections=added_sections,
                     added_test_cases=added_test_cases,
                 )
                 self.environment.log("\n".join(revert_logs))
                 exit(1)
 
+        if self.skip_run:
+            stop = time.time()
+            if added_test_cases:
+                self.environment.log(f"Submitted {len(added_test_cases)} test cases in {stop - start:.1f} secs.")
+            return
+        
+        # remove empty, unused sections created earlier, based on the sections actually used by the new test cases
+        #  - iterate on added_sections and remove those that are not used by the new test cases
+        empty_sections = None
+        if added_sections:
+            if not added_test_cases:
+                empty_sections = added_sections
+            else:
+                empty_sections = [section for section in added_sections if section['section_id'] not in [case['section_id'] for case in added_test_cases]]
+            if len(empty_sections) > 0:
+                self.environment.log("Removing unnecessary empty sections that may have been created earlier. ", new_line=False)
+                _, error = self.api_request_handler.delete_sections(empty_sections)
+                if error:
+                    self.environment.elog("\n" + error)
+                    exit(1)
+                else:
+                    self.environment.log(f"Removed {len(empty_sections)} unused/empty section(s).")
+
         # Create/update test run
         if not self.environment.run_id:
             self.environment.log(f"Creating test run. ", new_line=False)
             added_run, error_message = self.api_request_handler.add_run(
-                self.project.project_id, self.run_name, self.environment.milestone_id
+                self.project.project_id, self.run_name, self.environment.milestone_id, self.environment.plan_id,
+                self.environment.config_ids
             )
-            if error_message:
-                self.environment.elog("\n" + error_message)
-                revert_logs = self.rollback_changes(
-                    added_suite_id=added_suite_id,
-                    added_sections=added_sections,
-                    added_test_cases=added_test_cases,
-                )
-                self.environment.log("\n".join(revert_logs))
-                exit(1)
             run_id = added_run
-            self.environment.log(f"Run created: {self.environment.host.rstrip('/')}/index.php?/runs/view/{run_id}")
         else:
+            self.environment.log(f"Updating test run. ", new_line=False)
             run_id = self.environment.run_id
-            self.environment.log(f"Updating run: {self.environment.host.rstrip('/')}/index.php?/runs/view/{run_id}")
+            run, error_message = self.api_request_handler.update_run(
+                run_id, self.run_name, self.environment.milestone_id
+            )
+        if error_message:
+            self.environment.elog("\n" + error_message)
+            revert_logs = self.rollback_changes(
+                suite_id=suite_id,
+                suite_added=suite_added,
+                added_sections=added_sections,
+                added_test_cases=added_test_cases,
+            )
+            self.environment.log("\n".join(revert_logs))
+            exit(1)
+        else:
+            self.environment.log(f"Test run: {self.environment.host.rstrip('/')}/index.php?/runs/view/{run_id}")
+
         added_results, error_message, results_amount = self.api_request_handler.add_results(run_id)
         if error_message:
             self.environment.elog(error_message)
             revert_logs = self.rollback_changes(
-                added_suite_id=added_suite_id,
+                suite_id=suite_id,
+                suite_added=suite_added,
                 added_sections=added_sections,
                 added_test_cases=added_test_cases,
                 run_id=0 if run_id == self.environment.run_id else run_id,
@@ -146,7 +174,7 @@ class ResultsUploader:
         if results_amount:
             self.environment.log(f"Submitted {results_amount} test results in {stop - start:.1f} secs.")
 
-    def get_suite_id(self, project_id: int, suite_mode: int) -> Tuple[int, int]:
+    def get_suite_id(self, project_id: int, suite_mode: int) -> Tuple[int, int, bool]:
         """
         Gets and checks suite ID for specified project_id.
         Depending on the entry conditions (suite ID provided or not, suite mode, project ID)
@@ -154,17 +182,19 @@ class ResultsUploader:
             * check if specified suite ID exists and is correct
             * try to create missing suite ID
             * try to fetch suite ID from TestRail
-        Returns new suite ID if added or -1 in any other case. Proper information is printed
-        on failure.
+        Returns the suite ID if added/found or -1 in any other case, along with a return code, and a boolean
+        to identify is suite was created or not. Proper information is printed on failure.
         """
         suite_id = -1
         result_code = -1
+        suite_added = False
 
         if not self.api_request_handler.suites_data_from_provider.suite_id:
             if suite_mode in [SuiteModes.multiple_suites, SuiteModes.single_suite_baselines]:
                 suite_id, error_msg = self.api_request_handler.resolve_suite_id_using_name(project_id)
                 if suite_id != -1:
                     self.api_request_handler.suites_data_from_provider.suite_id = suite_id
+                    return suite_id, 1, False
             if suite_mode == SuiteModes.multiple_suites:
                 prompt_message = PROMPT_MESSAGES["create_new_suite"].format(
                     suite_name=self.api_request_handler.suites_data_from_provider.name,
@@ -183,6 +213,7 @@ class ResultsUploader:
                 )
                 if added_suites:
                     suite_id = added_suites[0]["suite_id"]
+                    suite_added = True
             elif suite_mode == SuiteModes.single_suite_baselines:
                 suite_ids, error_message = self.api_request_handler.get_suite_ids(
                     project_id=project_id
@@ -213,7 +244,7 @@ class ResultsUploader:
                 )
         else:
             result_code = self.check_suite_id(project_id)
-        return suite_id, result_code
+        return suite_id, result_code, suite_added
 
     def check_suite_id(self, project_id: int) -> int:
         """
@@ -231,7 +262,7 @@ class ResultsUploader:
             self.environment.elog(error_message)
         return result_code
 
-    def add_missing_sections(self, project_id: int) -> Tuple[list, int]:
+    def add_missing_sections(self, project_id: int) -> Tuple[List, int]:
         """
         Checks for missing sections in specified project. Add missing sections if user agrees to
         do so. Returns list of added section IDs if succeeds or empty list with result_code set to
@@ -344,7 +375,7 @@ class ResultsUploader:
         return api_client
 
     def rollback_changes(
-        self, added_suite_id=0, added_sections=None, added_test_cases=None, run_id=0
+        self, suite_id=0, suite_added=False, added_sections=None, added_test_cases=None, run_id=0
     ) -> List[str]:
         """
         Flow for rollback changes that was uploaded before error or user prompt.
@@ -365,7 +396,7 @@ class ResultsUploader:
                 returned_log.append(RevertMessages.run_deleted)
         if len(added_test_cases) > 0:
             _, error = self.api_request_handler.delete_cases(
-                added_suite_id, added_test_cases
+                suite_id, added_test_cases
             )
             if error:
                 returned_log.append(
@@ -381,8 +412,8 @@ class ResultsUploader:
                 )
             else:
                 returned_log.append(RevertMessages.section_deleted)
-        if self.project.suite_mode != SuiteModes.single_suite and added_suite_id > 0:
-            _, error = self.api_request_handler.delete_suite(added_suite_id)
+        if self.project.suite_mode != SuiteModes.single_suite and suite_added > 0:
+            _, error = self.api_request_handler.delete_suite(suite_id)
             if error:
                 returned_log.append(
                     RevertMessages.suite_not_deleted.format(error=error)
